@@ -446,32 +446,32 @@ class WordType(caching.base.CachingMixin,models.Model):
     def getNDs(self):
         if self.ND is not None and self.FWND is not None:
             return self.ND,self.FWND
-        possNeigh = []
-        segs = SegmentType.objects.all()
-        ur = self.getUR().split(";")
-        for i in xrange(len(ur)):
-            for j in xrange(len(segs)):
-                if ur[i] != segs[j].Label:
-                    neigh = ur[:i]
-                    neigh.append(segs[j].Label)
-                    neigh.extend(ur[i+1:])
-                    possNeigh.append(neigh)
-            neigh = ur[:i]
-            neigh.extend(ur[i+1:])
-            possNeigh.append(neigh)
-            for j in xrange(len(segs)):
-                neigh = ur[:i]
-                neigh.append(segs[j].Label)
-                neigh.extend(ur[i:])
-                possNeigh.append(neigh)
-        possNeigh = set([';'.join(x) for x in possNeigh])
-        neighCount = 0
-        fwnd = 0.0
-        words = WordType.objects.filter(~Q(Label__startswith="{"),~Q(Label__startswith="<"))
-        for w in words:
-            if w.getUR() in possNeigh:
-                neighCount += 1
-                fwnd += w.getFreq()
+        qs = WordType.objects.filter(Label__regex="$[^{<]").extra(
+                    select = {
+                        'ur': """(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
+                                FROM buckeyebrowser_underlying
+                                INNER JOIN buckeyebrowser_segmenttype
+                                ON buckeyebrowser_segmenttype.id
+                                    = buckeyebrowser_underlying.SegmentType_id
+                                WHERE buckeyebrowser_underlying.WordType_id
+                                    =buckeyebrowser_wordtype.id)"""})
+        word = qs.get(pk = self.pk)
+        patterns = []
+        for i in range(len(phones)):
+            patt = phones[:i] #Substitutions
+            patt.append(any_segment)
+            patt.extend(phones[i+1:])
+            patterns.append('^'+' '.join(patt) +'$')
+            patt = phones[:i] #Deletions
+            patt.extend(phones[i+1:])
+            patterns.append('^'+' '.join(patt) +'$')
+            patt = phones[:i] #Insertions
+            patt.append(any_segment)
+            patt.extend(phones[i:])
+            patterns.append('^'+' '.join(patt) +'$')
+        neighs = qs.filter(ur__regex = '|'.join(patterns)
+        nd = len(neighs)
+        fwnd = sum([ x.getFreq() for x in neighs])
         self.ND = neighCount
         self.FWND = fwnd
         self.save()
